// Libraries
#include <ESP8266WiFi.h>
#include <NTPClient.h>
#include <WiFiUdp.h>

// Constants
#define trigPin D7  // Trigger to Pin 7
#define echoPin D6  // Echo to Pin 6 
const int Starttime = 22; // start for watchdog
const int Endtime = 5;  // end for watchdog
const int watchdogpause = 3600000; // break in ms after alert was triggered
const int threshold = 40;  // distance threshold in cm

// Variables 
float duration, distance;
int currentHour; 

#include "arduino_secrets.h" // Secret keys and names

///////please enter your sensitive data in the Secret tab/arduino_secrets.h
char ssid[] = WLAN_SECRET_SSID;        // your network SSID (name)
char pass[] = WLAN_SECRET_PASS;    // your network password (use for WPA, or use as key for WEP)
String ifttt_webhook_key = SECRET_IFTTT_WEBHOOK_KEY; // your API key for the webhook
String ifttt_event = SECRET_EVENT;  // your name for the webhook event

const char* hostIftttMaker = "maker.ifttt.com"; // Website, where the webhook exists
String IftttMakerString = "";  // empty String variable for saving

WiFiUDP ntpUDP; // define NTP Client to get time
WiFiServer server(80);  // creating a server object

// You can specify the time server pool and the offset, (in seconds)
// additionaly you can specify the update interval (in milliseconds)
NTPClient timeClient(ntpUDP, "europe.pool.ntp.org", 3600, 60000);

void setup() {
  Serial.begin(115200); // opens serial port, sets data rate to 115200 bps
  delay(10);
  Serial.print("Startup reason:");
  Serial.println(ESP.getResetReason());

  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);

  startWiFi();

  server.begin();
  Serial.printf("Web server started, open %s in a web browser\n", WiFi.localIP().toString().c_str());
  Serial.print("Webserver is accessible between ");
  Serial.print(Starttime - 12);
  Serial.print(" PM and ");
  Serial.print(Endtime);
  Serial.println(" AM");
  delay(300000);  // wait five minutes before jumping into the loop

  timeClient.begin();
  
  Serial.print("Watchdog is ready for patrolling between... ");
  Serial.print(Starttime - 12);
  Serial.print(" PM and ");
  Serial.print(Endtime);
  Serial.println(" AM");
}

void loop() {
  timeClient.update();  // updating the internal clock through time server pool 
  delay(1000);
  Serial.println(timeClient.getFormattedTime()); // read from internal clock 
  int currentHour = timeClient.getHours(); // read hours from internal clock
  Serial.println(currentHour);

// deactivate the WiFi module to save power, when watchdog isn't active
if ((currentHour >= Endtime && currentHour < Starttime) && (WiFi.status() == WL_CONNECTED)) { 
    Serial.println("Internal clock was updated via time server pool");
    Serial.println("Set WiFi module to sleep mode till watchdog starting time");
    stopWiFiAndSleep();
}

// wake WiFi module up, when watchdog is active
  if ((currentHour >= Starttime || currentHour < Endtime) && (WiFi.status() != WL_CONNECTED)) { 
    Serial.println("Wake WiFi module up");
    startWiFiAndWake();
  }

  Serial.print("WiFi status: ");
  Serial.println(WiFi.status());

// Watchdog works only daily between starttime and endtime 
 if (currentHour >= Starttime || currentHour < Endtime) { 

// Write a pulse to the RCW-0001 Trigger Pin
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  
// Measure the response from the RCW-0001 Echo Pin
   duration = pulseIn(echoPin, HIGH);

// Determine distance from duration
// Use 343 meters per second as speed of sound
 
 distance = (duration * 0.0343) / 2;
    
// Send measurement to Serial Monitor
  Serial.print("Distance = ");
  Serial.println(distance);
  //delay(500);

  distanceRequest();  // executed, if client (web browser) connects

  // sensor can measure distances between 1 cm and 450 cm
  // alert threshold was a distance of 40 cm in my case
    if (distance > threshold) { // change the distance threshold
      Serial.println(distance);
      Serial.println("Intruder Detected!");
      Serial.println("Sending text notification..."); 
      Serial.print("Watchdog takes a break for ");
      Serial.print(watchdogpause/(1000*60));
      Serial.println(" minutes");
      SendIftttMaker(ifttt_event, ifttt_webhook_key);  // firing up the webhook
      delay(watchdogpause); // break before loop continues   
    }
    //delay(500);
   }
}

// function section
//***********************************************************

// function for starting WiFi connection
void startWiFi() {
  // Bring up the WiFi connection
  WiFi.begin(WLAN_SECRET_SSID, WLAN_SECRET_PASS);
  while (WiFi.status() != WL_CONNECTED)
  {
    delay(500);
    Serial.print(".");
  }
  Serial.println();
  Serial.print("WiFi Connected, IP address: ");
  Serial.println(WiFi.localIP());
}

// function for turning the WiFi off
void stopWiFiAndSleep() {
  WiFi.disconnect();
  delay(1000);
  WiFi.mode(WIFI_OFF);
  WiFi.forceSleepBegin();
  delay(1);
  Serial.println("WiFi module disabled...");
}

// function for wake up and starting WiFi connection
void startWiFiAndWake() {
  WiFi.forceSleepWake();
  delay(1);
  // Bring up the WiFi connection
  WiFi.mode(WIFI_STA);
  WiFi.begin(WLAN_SECRET_SSID, WLAN_SECRET_PASS);
  while (WiFi.status() != WL_CONNECTED)
  {
    delay(500);
    Serial.print(".");
  }
  Serial.println();
  Serial.print("WiFi Connected, IP address: ");
  Serial.println(WiFi.localIP());
}

// function for activating the webhook
void send2web(String thehost, String urlstring) {
  // Use WiFiClient class to create TCP connections
  WiFiClient client;
  const int httpPort = 80;
  if (!client.connect(thehost.c_str(), httpPort)) {
    Serial.println("Connection failed");
    Serial.println("Restart ESP");
    ESP.restart();
  }

  // We now create a URL for the request
  // This will send the request to the server
  client.print(String("GET ") + urlstring + " HTTP/1.1\r\n" +
               "Host: " + thehost + "\r\n" +
               "Connection: close\r\n\r\n");
  delay(4000);
  client.flush();
  client.stop();
  delay(100);
}

// Create url string and put host and url string into the function send2web 
void SendIftttMaker(String inIftttEvent, String inIftttMakerKey) {
  IftttMakerString = "/trigger/" + inIftttEvent + "/with/key/" + inIftttMakerKey;
  send2web(hostIftttMaker, IftttMakerString);
}

// function with web page content passed from server to client
 String prepareHtmlPage() {
  String htmlPage =
     String("HTTP/1.1 200 OK\r\n") +
            "Content-Type: text/html\r\n" +
            "Connection: close\r\n" +  // the connection will be closed after completion of the response
            "\r\n" +
            "<!DOCTYPE HTML>" +
            "<html>" +
            "Distance (Old Garage):  " + String(distance) + " cm" +
            "</html>" +
            "\r\n";
  return htmlPage;
}

// function for client request
void distanceRequest() {
  WiFiClient client = server.available();
  // wait for a client (web browser) to connect
  if (client) {
    Serial.println("\n[Client connected]");
    while (client.connected()) {
      // read line by line what the client (web browser) is requesting
      if (client.available()) {
        String line = client.readStringUntil('\r');
        Serial.print(line);
        // wait for end of client's request, that is marked with an empty line
        if (line.length() == 1 && line[0] == '\n')
        {
          client.println(prepareHtmlPage());
          break;
        }
      }
    }
    delay(1); // give the web browser time to receive the data

    // close the connection:
    client.stop();
    Serial.println("[Client disconnected]");
 }
}  
